﻿import { Injectable } from '@angular/core';
import { Http, Response, RequestOptionsArgs, RequestMethod, Headers } from '@angular/http';
import { Subject } from 'rxjs/subject';
import { Observable } from 'rxjs/observable'

@Injectable()
export abstract class HttpService {

	//Bindables
	private http: Http = null;

	/**
	 * Constructor
	 */
	public constructor(http: Http) {
		this.http = http;
	}

	/**
	 * Executes a POST on remote server for obtain a single object (using JSON response) 
	 * 
	 * @protected
	 * @template TResponse
	 * @param {string} fullUrl Full remote URL
	 * @param {Object} body Object for body request 
	 * @param {() => TResponse} createSingle Action for create single response instance
	 * @param {Map<string, string>} headers Custom headers
	 * @returns {Observable<TResponse>}
	 */
	protected postObject<TResponse>(fullUrl: string, body: Object, createSingle: () => TResponse,
		headers: Map<string, string> = null): Observable<TResponse> {

		//Arguments validation
		if (!fullUrl) {
			throw new Error("Argument 'fullurl' is invalid");
		}

		//Create observable source
		var observableSource = new Subject<TResponse>();

		//Declare observable
		var observable: Observable<TResponse> = observableSource.asObservable();

		//Use "raw" method
		this.rawPost(fullUrl, body, headers).subscribe(

			//Success
			(response) => {

				//Get JSON response
				var jsonObject = response.json();

				//Apply value on observable and complete
				observableSource.next(jsonObject);
				observableSource.complete();
			},

			//Error
			(error) => {

				//Apply error on observable and complete
				observableSource.error(error);
				observableSource.complete();
			}
		);

		//Returns observable
		return observable;
	}

	/**
	 * Executes a POST on remote server for obtain an array of objects (using JSON response) 
	 * 
	 * @protected
	 * @template TResponse
	 * @param {string} fullUrl Full remote URL
	 * @param {Object} body Object for body request
	 * @param {() => TResponse} createSingle Action for create single element instance on array response 
	 * @param {Map<string, string>} headers Custom headers
	 * @returns {Observable<TResponse[]>}
	 */
	protected postArray<TResponse>(fullUrl: string, body: Object, createSingle: () => TResponse,
		headers: Map<string, string> = null): Observable<TResponse[]> {

		//Arguments validation
		if (!fullUrl) {
			throw new Error("Argument 'fullurl' is invalid");
		}

		//Create observable source
		var observableSource = new Subject<TResponse[]>();

		//Declare observable
		var observable: Observable<TResponse[]> = observableSource.asObservable();

		//Use "raw" method
		this.rawPost(fullUrl, body, headers).subscribe(

			//Success
			(response) => {

				//Get JSON response
				var jsonArray = response.json();

				var outList: TResponse[] = [];

				//Iterate all elements on array
				for (var i = 0; i < jsonArray.length; i++) {

					//Create instance of element
					var instance: TResponse = createSingle();

					//Iterate all properties instance
					for (var key in instance) {

						//Set value of json on instance
						instance[key] = jsonArray[i][key];
					}

					//Push element on array
					outList.push(instance)
				}

				//Apply value on observable and complete
				observableSource.next(outList);
				observableSource.complete();
			},

			//Error
			(error) => {

				//Apply error on observable and complete
				observableSource.error(error);
				observableSource.complete();
			}
		);

		//Returns observable
		return observable;
	}

	/**
	 * Executes a POST on remote server for obtain an empty response
	 * 
	 * @protected
	 * @param {string} fullUrl Full remote URL
	 * @param {Object} body Object for body request
	 * @param {Map<string, string>} headers Custom headers
	 * @returns {Observable<void>}
	 */
	protected postEmpty(fullUrl: string, body: Object, headers: Map<string, string> = null): Observable<void> {

		//Arguments validation
		if (!fullUrl) {
			throw new Error("Argument 'fullurl' is invalid");
		}

		//Create observable source
		var observableSource = new Subject<any>();

		//Declare observable
		var observable: Observable<any> = observableSource.asObservable();

		//Use "raw" method
		this.rawPost(fullUrl, body, headers).subscribe(

			//Success
			(response) => {

				//Apply value on observable and complete
				observableSource.next(null);
				observableSource.complete();
			},

			//Error
			(error) => {

				//Apply error on observable and complete
				observableSource.error(error);
				observableSource.complete();
			}
		);

		//Returns observable
		return observable;
	}

	/**
	 * Executes a raw POST on remote server 
	 * 
	 * @private
	 * @param {string} fullUrl Full remote URL
	 * @param {Object} body Request body
	 * @param {Map<string, string>} headers Custom headers
	 * @returns {Observable<Response>}
	 */
	private rawPost(fullUrl: string, body: Object, headers: Map<string, string> = null): Observable<Response> {

		//Arguments validation
		if (!fullUrl) {
			throw new Error("Argument 'fullurl' is invalid");
		}

		//Create string for request
		var json = !body ? "" : JSON.stringify(body);

		//Create base headers
		var baseHeaders = new Headers({
			"Accept": "application/json",
			"Content-Type": "application/json; charset=utf-8"
		});

		//If headers were specified
		if (headers) {

			//Iterate custom headers and apply to base
			//headers, overwriting previous values
			headers.forEach((value, key) => {

				//Set value on base headers
				baseHeaders.set(key, value);
			});
		}

		//Compose request
		var options: RequestOptionsArgs = {
			url: fullUrl,
			method: RequestMethod.Post,
			search: null,
			headers: new Headers(baseHeaders),
			body: json
		};

		//Execute invoke
		return this.http.request(fullUrl, options);
	}
}
