﻿import { Injectable } from '@angular/core';

declare var toastr: any;

@Injectable()
export class UiUtilsService {

    /**
     * Constructor
     */
	constructor() {

		//Setting for toastr
		//This is for web notification
		toastr.options = {
			"progressBar": false,
			"positionClass": "toast-bottom-full-width",
			"showMethod": "slideDown",
			"hideMethod": "slideUp"
		};
	}

	/**
	* Raise ok feedback 
	*/
	public ok(): void {

		//Use toastr success
		toastr.success("L'operazione è stata completata con successo");
	}

    /**
     * Raise message for error
     */
	public auto(someError: any): void {

		//Select by error
		if (someError.status === 400) {
			toastr.warning("Alcuni dati immessi sono non corretti o mancanti");
		}
		else if (someError.status == 404) {
			toastr.warning("L'elemento richiesto non è stato trovato");
		}
		else if (someError.status == 401) {
			toastr.warning("L'operazione richiesta non è autorizzata");
		}
		else {
			toastr.error("Si è verificato un errore non specificato sul server");
		}
	}

    /**
     * Raise a warning message
     */
	public warning(message: string): void {
		toastr.warning(message);
	}

    /**
     * Raise an error message
     */
	public error(message: string): void {
		toastr.error(message);
	}

    /**
     * Raise an success message
     */
	public success(message: string): void {
		toastr.success(message);
	}

    /**
     * Raise an info message
     */
	public info(message: string): void {
		toastr.info(message);
	}

    /**
     * Converts a base64 string to Blob 
     * 
     * @param {string} b64Data Source string
     * @param {string} contentType File content type (optional)
     * @param {number} sliceSize Size of sliced data (optional)
     * @returns Returns blob instance
     */
	public base64toBlob(b64Data: string, contentType: string = null, sliceSize: number = null) {

		//Set content type if empty
		contentType = contentType || '';

		//Set slice size if not specified
		sliceSize = sliceSize || 512;

		let byteCharacters: string = atob(b64Data);
		var byteArrays: any[] = [];

		//Iterate all character
		for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {

			//Slice data inside array of chars and generate array
			var slice = byteCharacters.slice(offset, offset + sliceSize);
			var byteNumbers = new Array(slice.length);

			//Iterate char in "slice" and set char codes
			for (var i = 0; i < slice.length; i++) {
				byteNumbers[i] = slice.charCodeAt(i);
			}

			//Create array and push data
			var byteArray = new Uint8Array(byteNumbers);
			byteArrays.push(byteArray);
		}

		//Create blob property bag
		let bag: BlobPropertyBag = {
			type: contentType
		};

		//Create Blob and return
		var blob: Blob = new Blob(byteArrays, bag);
		return blob;
	}

    /**
     * Save provided Blob as downloaded file on browser  
     * 
     * @param {Blob} blob Blob instance
     * @param {string} fileName File name
     */
	public saveBlobAsFile(blob: Blob, fileName: string) {

		//Arguments validation
		if (!blob) {
			throw new Error("Argument 'blob' is invalid");
		}
		if (!fileName) {
			throw new Error("Argument 'fileName' is invalid");
		}

		//Create URL instance for current window
		let webKit: any = (window as any)["webkitURL"];
		let url: any = window.URL || webKit;

		//Create file URL for provided blob
		let fileURL: any = url.createObjectURL(blob);

		//Create dinamically "anchor" element
		let a: any = document.createElement("a");

		//Appen element to document body
		document.body.appendChild(a);

		//Set properties on anchor
		a.href = fileURL;
		a["download"] = fileName;

		//Execute "click" on anchor
		a.click();

		//Revoke url for blob
		url.revokeObjectURL(fileURL);

		//Remove inserted element from DOM 
		a.remove();
	}

    /**
     * Parse provided error response as "ModelState""
     * @param badRequestError Bad Request error
     */
	public parseModelState(badRequestError: any): string[] {

		//Arguments validation
		if (!badRequestError) {
			return [];
		}

        //If error has not "_body", exit
		if (!badRequestError._body) {
			return [];
		}

        //Try parse as JSON
		try {

            //Get object data
			var obj = JSON.parse(badRequestError._body);

            //If "modelState" does not exists, exit
			if (!obj.modelState) {
				return [];
			}

            //Check element "" of model state array
			if (!obj.modelState[""]) {
				return [];
			}

            //Get data
			return obj.modelState[""];
		}
		catch (exc){

            //Not JSON, just, return
			return [];
		}
	}
}
